import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import japanize_matplotlib
import scipy.stats as ss
import stan
import arviz as az
import seaborn as sb

finnm = input('入力データファイル（*.xlsx）= ')
txtoutnm = input('出力ファイル（*.txt）= ')
txtout = open(txtoutnm, 'w')
#
#      Excelファイルの読み込み
#
xlsx_fl = pd.ExcelFile(finnm)                       #   ExcelファイルからExcelFile型オブジェクトを生成
data_xlsx = pd.read_excel(xlsx_fl,  header = None)  #   ExcelFile型オブジェクトからDataFrame型オブジェクトを生成
data = data_xlsx.values                             #   Dataframe型オブジェクトからnumpyの配列オブジェクトを生成
print('\n', data)

txtout.write('\n入力データファイル: {}\n'.format(finnm))

VarNameA = data[0][1]
VarNameB = data[0][2]

dataA = []
dataB = []
for i in range(1,len(data)):
    if not pd.isnull(data[i][1]):
        dataA.append(data[i][1])
    if not pd.isnull(data[i][2]):
        dataB.append(data[i][2])
print('dataA =\n', dataA)
print('dataB =\n', dataB)

txtout.write('\n{0}\n'.format(VarNameA))
txtout.write('{0}\n'.format(dataA))
txtout.write('\n{0}\n'.format(VarNameB))
txtout.write('{0}\n'.format(dataB))

meanA = np.mean(dataA)
meanB = np.mean(dataB)
sdA = np.std(dataA)
sdB = np.std(dataB)
print(VarNameA)
print('平均値 = {0:.2f}　　標準偏差 = {1:.2f}'.format(meanA, sdA))
print(VarNameB)
print('平均値 = {0:.2f}　　標準偏差 = {1:.2f}'.format(meanB, sdB))
txtout.write('\n' + VarNameA + '\n')
txtout.write('平均値 = {0:.2f}　　標準偏差 = {1:.2f}\n'.format(meanA, sdA))
txtout.write('\n' + VarNameB + '\n')
txtout.write('平均値 = {0:.2f}　　標準偏差 = {1:.2f}\n'.format(meanB, sdB))

plt.title(('{0}： mean = {1:.2f}  sd = {2:.2f}'.format(VarNameA, meanA, sdA)) + \
          '\n{0}： mean = {1:.2f}  sd = {2:.2f}'.format(VarNameB, meanB, sdB),
          fontsize = 16)
plt.hist(dataA, alpha = 0.5, label = VarNameA)
plt.hist(dataB, alpha = 0.5, label = VarNameB)
plt.xlabel('値', fontsize = 14)
plt.ylabel('度数', fontsize = 14)
plt.legend()
plt.tight_layout()
plt.show()

Data = {'n1':len(dataA), 'x1':dataA, 'n2':len(dataB), 'x2':dataB,
        'dmean1':meanA, 'dsgm1':sdA, 'dmean2':meanB, 'dsgm2':sdB}
with open('diff_model.stan') as f_stan:
    sm = stan.build(f_stan.read(), data=Data)
fit = sm.sample()

i_data = az.from_pystan(posterior=fit, posterior_model=sm)

smry = az.summary(i_data)      #     Summary of statistics of MCMC sampling
print('Summary...\n', smry)
txtout.write(f'\nSummary...\n{smry}\n')
#
#         Trace graph of MCMC sampling
#
az.plot_trace(i_data, var_names=['mu1', 'mu2', 'sgm1', 'sgm2'],
              figsize=(10,7))
plt.tight_layout()
plt.show()
#
#         Posterior distributions of parameters mu1 and mu2
#
d_frame = fit.to_frame()
mu1_para = d_frame['mu1']
mu2_para = d_frame['mu2']
q1A, q2A, q3A = np.percentile(mu1_para, [25, 50, 75])
q1B, q2B, q3B = np.percentile(mu2_para, [25, 50, 75])
strA = f'{VarNameA}:  Q1 = {q1A:.2f}, Q2 = {q2A:.2f}, Q3 = {q3A:.2f}\n' + \
       f'{VarNameB}:  Q1 = {q1B:.2f}, Q2 = {q2B:.2f}, Q3 = {q3B:.2f}'
sb.kdeplot(mu1_para, color='g', label=VarNameA)
sb.kdeplot(mu2_para, color='b', label=VarNameB)
plt.xlabel('mu1 or mu2', fontsize=16)
plt.title(strA)
plt.legend()
plt.show()
txtout.write('\nPosterior distributions of mu1 and mu2\n' + strA + '\n')
#
#           Posterior distribution of difference mu1 - mu2
#
diff_para = d_frame['diff']
print(diff_para)
diff_para = np.array(diff_para)
print(diff_para)
q1D, q2D, q3D = np.percentile(diff_para, [25, 50, 75])
LvUv = az.hdi(diff_para, hdi_prob=0.95)
print('LvUv =', LvUv)
strD = f'mu1-m2: Q1 = {q1D:.2f}, Q2 = {q2D:.2f}, Q3 = {q3D:.2f}\n' + \
      f'95% HDI = [{LvUv[0]:.2f}, {LvUv[1]:.2f}]'
txtout.write('\nPosterior distribution of mu1 - mu2...\n' + strD + '\n')
sb.kdeplot(diff_para)
plt.title(strD)
plt.xlabel('mu1-m2', fontsize = 16)
plt.tight_layout()
plt.show()
#
#           Posterior distributions of parameters sgm1 and sgm2
#
sgm1_para = d_frame['sgm1']
sgm2_para = d_frame['sgm2']
q1DA, q2DA, q3DA = np.percentile(sgm1_para, [25, 50, 75])
q1DB, q2DB, q3DB = np.percentile(sgm2_para, [25, 50, 75])
strSgm = f'{VarNameA}:  Q1 = {q1DA:.2f}, Q2 = {q2DA:.2f}, Q3 = {q3DA:.2f}\n' + \
       f'{VarNameB}:  Q1 = {q1DB:.2f}, Q2 = {q2DB:.2f}, Q3 = {q3DB:.2f}'
txtout.write('\nPosterior distributions of sgm1 and sgm2...\n' + strSgm + '\n')
sb.kdeplot(sgm1_para, label=VarNameA)
sb.kdeplot(sgm2_para, label=VarNameB)
plt.xlabel('sgm1 or sgm2', fontsize=16)
plt.title(strSgm)
plt.legend()
plt.tight_layout()
plt.show()
#
#          Posterior distribution of difference sgm1 - sgm2
#
dsgm_para = d_frame['dsgm']
dsgm_para = np.array(dsgm_para)
q1DS, q2DS, q3DS = np.percentile(dsgm_para, [25, 50, 75])
LvUvS = az.hdi(dsgm_para, hdi_prob=0.95)
print('LvUvS =', LvUvS)
strD = f'sgm1-sgm: Q1 = {q1DS:.2f}, Q2 = {q2DS:.2f}, Q3 = {q3DS:.2f}\n' + \
      f'95% HDI = [{LvUvS[0]:.2f}, {LvUvS[1]:.2f}]'
txtout.write('\nPosterior distribution of sgm1 - sgm2...\n' + strD + '\n')
sb.kdeplot(dsgm_para)
plt.title(strD)
plt.xlabel('sgm1 - sgm2', fontsize=16)
plt.legend()
plt.tight_layout()
plt.show()


txtout.close()
print(f'\n{txtoutnm} was saved.\n')
